.g-item-slicer-ui
  margin-top 20px

.s-widget-slicer-cli-run-container
  .s-panel
    margin 2px 5px
    border 1px solid #f0f0f0

  .s-panel-title-container
    font-size 120%
    margin-top 5px 0
    display flex
    cursor pointer
    background-color #f0f0f0
    color #555

  .s-panel-title
    order 2
    margin-left 5px

  .s-panel-content
    margin-left 10px
