.s-control-widget-label
  margin-bottom 2px

.s-control-widget-description
  font-size 12px
  color #666
  margin-bottom 4px

.s-control-item
  input.form-control
    height auto
  .checkbox label
    font-weight 700

.s-select-multifile-button
  span.fa-stack
    i.fa-cancel-cover
      color inherit

.has-error
  .s-control-widget-label::before,
  .checkbox::before,
  .radio::before,
  .checkbox-inline::before,
  .radio-inline::before
    color #a94442
    content "\e101"
    font-family "Glyphicons Halflings"
    margin-right 5px
    vertical-align text-top
  .form-control
    background-color #f2dede
    border-color #ebccd1
    color #a94442
    ::placeholder
      color #a94442

.btn-group.region-dropdown
  button.dropdown-toggle
    padding-left 2px
    padding-right 2px
  .dropdown-menu
    left inherit
    right 0
    padding 0
    min-width 0
    button.btn
      display block
      text-align left
      width 100%
      padding-right 18px
