.s-panel-group

  .s-panel
    background-color rgba(255, 255, 255, 0.95)
    margin-top 5px
    border-radius 3px
    height auto
    border 1px solid #d6d6d6

  .s-panel-content
    margin 5px

  .s-panel-content:empty
    margin 0

  .s-panel-title-container
    font-weight bold
    text-align left
    background-color #f0f0f0
    cursor pointer
    padding 5px

    .s-panel-title
      display inline-block
      max-width 85%
      overflow hidden

    .s-panel-controls
      float right

.s-panel
  .g-jobs-list-table tbody>tr td
    white-space nowrap
    a
      max-width 235px
      text-overflow ellipsis
      overflow hidden
      display block
